#!/usr/bin/env python
# Copyright 2006, CRIM, Revolution Linux Inc.
#
# $Id: initrd.py 1340 2006-03-17 20:26:29Z francis $
#
# Authors : Francis Giraldeau <francis.giraldeau@revolutionlinux.com>
#
# This file is part of the MILLE-XTERM project.
#
#		http://www.revolutionlinux.com/mille-xterm/
#		http://www.mille.ca/
#
# This program is covered by the GNU General Public License.
# See the COPYING file in the top-level MILLE-XTERM directory.
#
# -------------------------------------------------------------------------

"""
Script to auto-configure the bootserver
"""

from inisettings import *
import os
import sys
import re
import glob
import shutil
import string
import socket
import fcntl
import struct
from copy import copy
from subprocess import *
from utils import *

def get_ip_address(ifname):
	s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
	return socket.inet_ntoa(fcntl.ioctl(
		s.fileno(),
		0x8915,  # SIOCGIFADDR
		struct.pack('256s', ifname[:15])
	)[20:24])

xtroot = "/opt/xtermroot"

def backup(file):
	if not os.path.isfile(file):
		print "Can't backup : is not a file"
		return
	num = string.zfill(len(glob.glob(file + "*")),3)
	shutil.copy2(file, file + "." + str(num))

class NfsServerFactory:
	def __init__(self):
		pass

	def testRpmPackage(self, package):
		query = Popen(['rpm','-q',package])
		retcode = query.wait()
		if retcode == 0: return True
		else: return False

	def getNfsServer(self):
		# Detect the server installed
		# and return the right instance
		nfs_server = None
		
		if self.testRpmPackage('unfs3'):
			nfs_server = UserSpaceMandriva2006NfsServer() 
			
		if self.testRpmPackage('nfs-utils'):
			nfs_server = Mandriva2006NfsServer()
		
		return nfs_server

class Mandriva2006NfsServer:
	def __init__(self):
		self.need_restart = False
		self.scripts = ['/etc/init.d/portmap','/etc/init.d/nfslock','/etc/init.d/nfs']
		self.cmds = {'add_service':['chkconfig','--add']}
		self.nfs_export_file = "/etc/exports"
		self.nfs_syscon_file = "/etc/sysconfig/nfs"
		self.steps = ['configNfsExport','configNfsOptions','enableNfs']
	
	def configure(self):
		for step in self.steps:
			eval("self." + step)()

	def enableNfs(self):
		if self.need_restart:
			for script in self.scripts:
				if os.path.isfile(script):
					print "Using : ", script
					cmd_add_service = copy(self.cmds['add_service'])
					cmd_add_service.append(os.path.basename(script))
					execute_cmd(cmd_add_service)
					execute_cmd([script,"restart"])

	def configNfsExport(self):
		"""configuration of the NFS exports and server options"""
		###
		### /etc/export
		###
		
		nfs_export_file = self.nfs_export_file

		# backup the config file before anything
		backup(nfs_export_file)
		
		lines = []
		if os.path.isfile(nfs_export_file):
			file = open(nfs_export_file,"r")
			lines = file.readlines()
			file.close()

		found = False
		export_line = xtroot + "\t" + "0.0.0.0/0.0.0.0(ro,no_root_squash,async) # MILLE-XTERM\n"
		for i,line in enumerate(lines):
			if "# MILLE-XTERM" in line:
				if line == export_line:
					print "File %s already configured" % nfs_export_file
					found = True
					break
				else:
					print "modify file : " + nfs_export_file
					lines[i] = export_line
					found = True
					break
		if not found:
			print "modify file : " + nfs_export_file
			lines.append(export_line)

		file = open(nfs_export_file,"w+")
		file.writelines(lines)
		file.close()
		self.need_restart = True

	def configNfsOptions(self):
		###
		### /etc/sysconfig/nfs
		###
		nfs_syscon_file = self.nfs_syscon_file

		backup(nfs_syscon_file)
		lines = []
		if os.path.isfile(nfs_syscon_file):
			file = open(nfs_syscon_file,"r")
			lines = file.readlines()
			file.close()

		conf = {}
		conf["MOUNTD_OPEN_FILES"] = "256"
		conf["NFSDCOUNT"] = "16"
		conf["TUNE_QUEUE"] = "yes"
		conf["NFS_QS"] = "262144"

		for k,v in conf.items():
			found = False
			nfs_conf_line = k + "=" + v + "\n"
			for i, line in enumerate(lines):
				if k + "=" in line:
					lines[i] = nfs_conf_line
					found = True
					print "modify file : " + nfs_syscon_file
			if not found :
				lines.append(nfs_conf_line)
				print "modify file : " + nfs_syscon_file

		file = open(nfs_syscon_file,"w+")
		file.writelines(lines)
		file.close
		self.need_restart = True

		
class UserSpaceMandriva2006NfsServer(Mandriva2006NfsServer):
	def __init__(self):
		Mandriva2006NfsServer.__init__(self)
		self.scripts = ['/etc/init.d/portmap','/etc/init.d/unfs3']

	def configNfsOptions(self): 
		"""Not implemented with unfs3"""
		pass
		

# configuration of pxe, tftpd and dhcp if enabled

def SelectTerminalConfigurationSource(ini_settings):
	"""Define config server in the xtermroot"""
	settings = ini_settings.settings
	ini_settings.setDefault("global","config_server_name","")
	ini_settings.setDefault("global","use_config_server","")
	ini_settings.save()

	want_config_server = string.upper(settings['global']['use_config_server'])
	if 'YES' not in want_config_server or 'NO' in want_config_server:
		print "Terminals needs a configuration source."
		print "With mille-xterm, you can use the legacy lts.conf file,"
		print "or the web based configuration management with mille-xterm-configurator."
		q = "Wich one will be used?"
		choices = ['Web based configuration (configurator)','File based configuration (lts.conf)']
		choice = AskChoice(q, choices)
		if choice == 0:
			settings['global']['use_config_server'] = "yes"
		elif choice == 1:
			settings['global']['use_config_server'] = "no"
		else:
			raise RuntimeError, "Shouldn't have reach this condition"
		ini_settings.save()

	config_server_file = os.path.join(xtroot,'etc','config-server')
	if string.upper(settings['global']['use_config_server']) == "YES":
		print "Terminals read the URL of the configurator in the file ", config_server_file
		print "We are about to configure this file"
		if settings['global']['config_server_name'] == '':
			addr = AskAddress('Enter the hostname or the ip address of the configurator.')
			settings['global']['config_server_name'] = addr
		else:
			if AskYesNo('Does this URL is correct ? \n\n' + settings['global']['config_server_name'] + '\n'):
				if not testUrl(settings['global']['config_server_name']):
					print "This address doesn't ping. Please provide a valid configurator hostname or address"
					addr = AskAddress('Enter the hostname or the ip address of the configurator.')
					settings['global']['config_server_name'] = addr
			else:
					addr = AskAddress('Enter the hostname or the ip address of the configurator.')
					settings['global']['config_server_name'] = addr
		ini_settings.save()		
		wise_mkdir(os.path.dirname(config_server_file))
		print "Writing ", config_server_file
		file = open(config_server_file,"w+")
		file.write("http://" + settings['global']['config_server_name'] + "/mille-configurator/Terminal/")
		file.close()
		

	if string.upper(settings['global']['use_config_server']) == "NO":
		print "You will have to configure manually the file ", os.path.join(xtroot,'etc','lts.conf')
	

def testUrl(host):
	print "Testing : ", host
	if os.system("ping -c 2 " + host ) == 0:
		return True
	return False

def AskYesNo(question):
	print "\n  " + str(question) + "\n"
	ans = string.strip(raw_input("(y/n) ? "))
	ans = string.upper(ans)
	if ans == 'Y' or ans == 'YES':
		return True
	elif ans == 'N' or ans == 'NO':
		return False
	else:
		print 'Please select "yes" or "no" '
		return AskYesNo(question)
	
def AskChoice(question, choices):
	print "\n  " + str(question) + "\n"
	select = []
	for i, choice in enumerate(choices):
		print '[%s] %s' % (i,choice)
		select.append(str(i))

	select_info = '(' + string.join(select,'/') + ') ? ' 
	
	ans = string.strip(raw_input(select_info))
	if ans in select:
		return int(ans)
	else:
		print 'Please select un number in the list '
		return AskChoice(question, choices)

def AskAddress(question):
	print "\n  " + str(question) + "\n"
	ans = string.strip(raw_input("(adress or hostname) ? "))
	if testUrl(ans):
		return ans
	else:
		print 'The server is not responding, try again. '
		return AskAddress(question)

def AskSlaveList():
	print '\n  Enter ip address or hostname of slave bootserver '
	ans = [string.strip(raw_input("(adress or hostname) ? "))]
	while AskYesNo('Another bootserver slave ?'):
		print '\n  Enter ip address or hostname of slave bootserver '
		ans.append(string.strip(raw_input("(adress or hostname) ? ")))
	if AskYesNo('Does this list is correct ? ' + "\n\n" + string.join(ans) + "\n"):
		return ans
	else:
		print 'Ok. Try again.'
		return AskSlaveList()
	
def defineRole(ini_settings):
	#
	# /etc/xinetd.d/rsync
	#
	settings = ini_settings.settings
        ini_settings.setDefault("global","role","")
        ini_settings.save()

	q = "Select a role for this bootserver : "
	choices = ['Master : will offer the xtermroot by rsync', \
		   'Slave  : will get the xtermroot by rsync from a master', \
		   'Standalone : do not rsync anything' ]
	num = AskChoice(q, choices)
	
	if num == 0:
		settings['global']['role'] = 'master'
		SelectTerminalConfigurationSource(ini_settings)
		configRsyncMaster(ini_settings)
	elif num == 1:
		settings['global']['role'] = 'slave'
		configRsyncSlave(ini_settings)
	elif num == 2:
		settings['global']['role'] = 'standalone'
	else:
		raise RuntimeError, "Shouldn't have reach this condition"

	ini_settings.save()

def configRsyncMaster(ini_settings):
	# Enable rsync server in xinetd
	settings = ini_settings.settings
        ini_settings.setDefault("global","slave_list","")
	ini_settings.save()

	xinetd_rsync_config = '/etc/xinetd.d/rsync'
	backup(xinetd_rsync_config)
	file = open(xinetd_rsync_config)
	lines = file.readlines()
	file.close()

	for i, line in enumerate(lines):
		if 'disable' in line:
			lines[i] = '        disable = no\n'
			break
	file = open(xinetd_rsync_config,'w+')
	file.writelines(lines)
	file.close()
	execute_cmd(['/etc/init.d/xinetd','restart'])

	if settings['global']['slave_list'] == '':
		slave_list = AskSlaveList()
		settings['global']['slave_list'] = string.join(slave_list,',')
	else:
		if not AskYesNo("Does the following list is correct? \n\n" + settings['global']['slave_list']):
			print "Ok. Start over"
			slave_list = AskSlaveList()
			settings['global']['slave_list'] = string.join(slave_list,',')

	ini_settings.save()

	# Configure rsync servers parameters
	hosts_allow = string.join( string.split(settings['global']['slave_list'],',') ,' ')

	rsyncd_config = '/etc/rsyncd.conf'
	backup(rsyncd_config)
	# Remove the config file and build it
	# We can't read the config file with IniSettings
	# Because it doesn't support configuration entries without section
	if os.path.isfile(rsyncd_config):
		os.remove(rsyncd_config)
		
	ini = IniSettings(rsyncd_config)
	conf = ini.settings
	conf['uid'] = 'root'
	conf['gid'] = 'root'
	conf['use chroot'] = 'no'
	conf['max connections'] = '10'
	conf['syslog facility'] = 'local5'
	conf['pid file'] = '/var/run/rsyncd.pid'
	conf['mxboot-server-master'] = {}
	conf['mxboot-server-master']['path'] = xtroot 
	conf['mxboot-server-master']['comment'] = 'Mille-xterm bootserver xtermroot' 
	conf['mxboot-server-master']['read only'] = 'yes'
	conf['mxboot-server-master']['ignore errors'] = 'true'
	conf['mxboot-server-master']['ignore nonreadable'] = 'true'
	conf['mxboot-server-master']['list'] = 'true'
	conf['mxboot-server-master']['hosts allow'] = hosts_allow
	ini.save()

def configRsyncSlave(ini_settings):
	settings = ini_settings.settings
        ini_settings.setDefault("global","master_url","")
	if settings['global']['master_url'] == '':
		addr = AskAddress('What is the hostname or ip address of the master bootserver ? ')
		settings['global']['master_url'] = addr
	else:
		if not AskYesNo("Does this master bootserver address is correct ? " + settings['global']['master_url']):
			addr = AskAddress('What is the hostname or ip address of the master bootserver ? ')
			settings['global']['master_url'] = addr

	if AskYesNo("Rsync server is not needed on a bootmaster slave. Do you want to disable it ?"):
		xinetd_rsync_config = '/etc/xinetd.d/rsync'
		backup(xinetd_rsync_config)
		file = open(xinetd_rsync_config)
		lines = file.readlines()
		file.close()
		# FIXME: of no 'disable' line is found, we should reconfigure the file
		# or warn the admin about that
		for i, line in enumerate(lines):
			if 'disable' in line:
				lines[i] = '        disable = yes\n'
				break
		file = open(xinetd_rsync_config,'w+')
		file.writelines(lines)
		file.close()
		execute_cmd(['/etc/init.d/xinetd','restart'])
	
def wizard():
        print "\nStarting configuration wizard.\n"
	nfs_server_factory = NfsServerFactory()
	nfs_server = nfs_server_factory.getNfsServer()
	if not nfs_server:
		print "Error : no nfs server available"
		sys.exit(1)

	ini_settings = IniSettings('/etc/mille-xterm/bootserver.ini')

	defineRole(ini_settings)
	
	nfs_server.configure()

	print ""
        print "Configuration complete.\n"
	print ""
        print "Don't forget to define the list of application servers in /etc/mille-xterm/lbsconfig.xml"
	print ""

	role = ini_settings.settings['global']['role']
	print "* Because you selected the role ", role
	if role in ['master','standalone']:
		print "* You must run the command 'mille-xterm-bootserver build_xtermroot"
	if role == 'slave':
		print "* You must run the command 'mille-xterm-bootserver sync"
	print ""

def main():
	wizard()
	
if __name__=="__main__":
	try:
		main()
	except KeyboardInterrupt:
		print "\nQuit."
		sys.exit(1)
