#!/usr/bin/env python
# Copyright 2006, CRIM, Revolution Linux Inc.
#
# $Id: initrd.py 1142 2006-02-24 16:06:53Z francis $
#
# Authors : Francis Giraldeau <francis.giraldeau@revolutionlinux.com>
#
# This file is part of the MILLE-XTERM project.
#
#		http://www.revolutionlinux.com/mille-xterm/
#		http://www.mille.ca/
#
# This program is covered by the GNU General Public License.
# See the COPYING file in the top-level MILLE-XTERM directory.
#
# -------------------------------------------------------------------------

import os, sys
import string
import time
from subprocess import *

from initrd import sh, busybox_utilities, copytree2 
from utils import wise_mkdir

debug = 1 
dry_run = 0

urpmi_ret_code = {1:'Command line inconsistency.',
		  2:'Problem registering local packages.',
		  3:'Source packages not retrievable.',
		  4:'Medium is not selected.',
		  5:'Medium already exists.',
		  6:'Unable to save configuration.',
		  7:'Urpmi database locked.',
		  8:'Unable to create bug report.',
		  9:'Unable to open rpmdb.',
		  10:'Some files are missing for installation.',
		  11:'Some transactions failed but not all.',
		  12:'All transactions failed.',
		  13:'Some files are missing and some transactions failed but not all.',
		  14:'Some files are missing and all transactions failed.'}


def mklink(root,target,link,force=False):
	"""
	Create relative links that points to target, into a given root
	Ex : link("/bin/busybox","/usr/bin/ping") will do : ln -s ../../bin/busybox usr/bin/ping
	"""
	tmp_path = os.path.abspath(os.path.curdir)
	os.chdir(root)

	# Remove / from the link if any
	if len(link) == 0 or len(target) == 0:
		print "Target or link empty : target=",target," link=",link
		return

	if link[0] == os.path.sep:
		link = link[1:]

	# Remove / from the target if any
	if target[0] == os.path.sep:
		target = target[1:]
		
	# Test if the target exists
	if not os.path.exists(target):
		print "Error : not a file : ", os.path.join(root,target)
		os.chdir(tmp_path)
		return

	# Test if the link exists, and erase it if force is True
	if os.path.exists(link):
		if force is True:
			if debug : print "Message : link ", os.path.join(root,link) ," already exists, but removing is forced"
			sh(['rm','-rf',os.path.join(root,link)])
		else :
			print "Error : file already exists and force is false : ", os.path.join(root,link)
			os.chdir(tmp_path)
			return
		
	# Create the link directory if it doesn't exists
	link_dir, link_base = os.path.split(link)
	wise_mkdir(link_dir)

	# Get the deep of the link, and add .. to the target
	dir_target, basename = os.path.split(target)
	#split_dir_target = string.split(dir_target,os.path.sep)

	dir_link, basename = os.path.split(link)
	split_dir_link = string.split(dir_link,os.path.sep)

	#deep_target = len(string.split(dir_target,os.path.sep))
	deep_link = len(string.split(dir_link,os.path.sep))

	if split_dir_link[0] is not "":
		for i in range(deep_link):
			target = os.path.join(os.pardir,target)
	
	# Do the link
	try : 
		if debug: print "Linking : target=", target, " link=", link 
		os.symlink(target,link)
	except :
		if debug : print "Error : ", str(sys.exc_value)

	os.chdir(tmp_path)

def create_file(file):
	d, f = os.path.split(file)
	#print "Create file ", f, " in directory ", d 
	wise_mkdir(d)
	sh(["touch",file])

def create_devices(devices, dev_dir):
	wise_mkdir(dev_dir)
	for dev in devices.keys():
		dev_path = os.path.join(dev_dir,dev)
		sh(["mknod",dev_path,devices[dev]])
		if dev == "null":
			sh(["chmod","666",dev_path])
def build():

	if os.getuid() is not 0:
		print "You must be root to run this script."
		sys.exit(1)

	print 'Starting the build of the xtermroot.'


	# FIXME : global path variables should be put into the __init__.py file
	package = "mille-xterm-bootserver"
	lib_dir = "/var/lib"
	urpmi_cmd=["/usr/sbin/urpmi","--no-verify-rpm","--auto","--excludedocs","--noclean"]
	xtroot="/opt/xtermroot"
	# FIXME : should be an option into the bootserver.ini
	rpmlist_name=os.path.join(lib_dir,package,"rpmlist.mdv2006")
	devel_rpmlist_name =  "rpmlist.mdv2006"
	if os.path.isfile(devel_rpmlist_name):
		rpmlist_name = devel_rpmlist_name

	#ramdir="/tmp"
	xtermtree = os.path.join(lib_dir,package,"xtermroot")
	# Fallback to relative directory for devel
	devel_xterm_tree = "data/xtermroot"
	if os.path.isdir(devel_xterm_tree):
		xtermtree = devel_xterm_tree
	
	# Check the coherence of paths. If a path is for devel, the other one must be too
	if (devel_rpmlist_name == rpmlist_name or xtermtree == devel_xterm_tree):
		if not (devel_rpmlist_name == rpmlist_name and xtermtree == devel_xterm_tree):
			print "Error : rpmlist and xtermroot are not both a devel path"
			print "rpmlist_name=%s" % rpmlist_name
			print "xtermtree=%s" % xtermtree
		
	os.putenv("LC_ALL", "C")

	# Verify that xtroot is not at the system root
	# Because, the script may erase important things
	d, f = os.path.split(xtroot)
	if d is "":
		print "Error : xtroot must not be the system root"
		sys.exit(1)
		
	timestamp = "" 
	for i in time.gmtime()[:6]:
		timestamp += str(i)

	# We can use a partially installed xtermroot
	#if os.path.isdir(xtroot):
	#	destdir = xtroot + ".save."+timestamp
	#	print "$xtroot already exists, moving to ", destdir
	#	sh(["mv","-f",xtroot,destdir])

	# Create directories
	dirs = [ "/var/lib/rpm", 	\
			"/dev",		\
			"/bin",		\
			"/home",	\
			"/lib/modules",	\
			"/mnt",		\
			"/oldroot",	\
			"/proc",	\
			"/sys",		\
			"/root/tmp",	\
			"/tmp/home",	\
			"/sbin",	\
			"/usr/bin",	\
			"/usr/sbin",	\
			"/var/lock/subsys",	\
			"/etc/sysconfig" ]		
	for dir in dirs :
		dir = dir[1:]
		d = os.path.join(xtroot,dir)
		sh(["mkdir","-p",d])

	# Create minimal devices
	devices = {"null":"c 1 3", "urandom":"c 1 9"} 
		#   "ubd0":"b 98 0",
		#   "tty1": "c 4 1",
		#"full": "c 1 7",
		#"ptmx": "c 5 2",
		#"random": "c 1 8",
		#"tty": "c 5 0",
		#"urandom": "c 1 9",
		#"zero": "c 1 5" }
	create_devices(devices, os.path.join(xtroot,"dev"))
	
	# Create files 
	files = ["etc/ld.so.conf","etc/fstab","etc/shadow"]
	for file in files:
		create_file(os.path.join(xtroot,file))

	# Install packages
	# for each package in file, concatenate into one line
	packages = []
	file = open(rpmlist_name,"r")
	lines = file.readlines()
	for line in lines:
		line = string.strip(line)
		if len(line) > 0:
			if line[0] != '#':
				packages.append(line)
	failed = True
	if not dry_run:
		try:
			cmd = urpmi_cmd + ["--root",xtroot] + packages
			log_file = open('/opt/xtermroot.err','w')
			urpmi_process = Popen(cmd, stderr=log_file)
			retcode = urpmi_process.wait()
			if retcode < 0:
				print >>sys.stderr, "Child was terminated by signal ", -retcode
			elif retcode > 0:
				print >>sys.stderr, "URPMI exited with error : %s" % urpmi_ret_code.get(retcode, retcode)
			else:
				print >>sys.stderr, "URPMI installed packages successfully", retcode
				failed = False
		except OSError, e:
			print >>sys.stderr, "Execution failed:", e
			sys.exit(1)
		except KeyboardInterrupt, e:
			print >>sys.stderr, "Keybord interrupt, the xtermroot is not completed. You must run the command again."
			sys.exit(1)

	if failed:
		print "An error occured. Common causes are : "
		print " - Low disk space"
		print " - Misconfigured urpmi sources"
		
	time.sleep(2)

	# Copy all LTSP tree into the new root
	# Note : work has been done to do a RPM with the root, but
	# files conflicts with initscripts : rc.sysinit, rc.local
	
	copytree2(xtermtree,xtroot)

	# Deprecated since the use of Unionfs

	# Move etc to etc.local and etc.local/X11 to etc.static/X11
	# We do that because hal fstab-sync needs to write into /etc directly
	# To save space in ramdisk, we leave X11 in /etc.static
	#tmpdir = os.path.abspath(".")
	#os.chdir(xtroot)
	#sh(["mv","etc","etc.local"])
	#sh(["ln","-s","etc.local","etc"])
	#sh(["mkdir","etc.static"])
	#sh(["mv","etc.local/X11","etc.static/"])
	#os.chdir(tmpdir)
	
	# Do symlinks to busybox
	# Busybox is lighter than similar tools
	#needed_utils = ["/bin/rm","/bin/ping","/bin/ps","/bin/sed","/bin/wget","/bin/udhcpc",\
	#	 "/sbin/freeramdisk","/sbin/loadkmap","/sbin/loadkmap","/usr/bin/basename",\
	#	 "/usr/bin/free","/usr/bin/id","/usr/bin/loadfont","/usr/bin/nslookup","/usr/bin/sort",\
	#	 "/usr/bin/telnet","/usr/bin/traceroute","/usr/bin/uptime","/usr/bin/xargs","/usr/bin/find",\
	#	 "/usr/bin/vi","/sbin/getty","/sbin/init","/bin/[","/bin/mount","/bin/umount","/bin/echo"]
	#	 #"/usr/bin/vi","/sbin/getty"]
	#bp = os.path.join(xtroot,"bin","busybox")
	#all_utils = busybox_utilities(bp)
	#if debug : print needed_utils
	#for util in needed_utils:
	#	d, u = os.path.split(util)
	#	if u not in all_utils:
	#		print "Warning : util ", u, " not compiled in busybox, skipping"
	#	else : 
	#		pass
	#		#mklink(xtroot,"/bin/busybox",util,force=True)
		
	#ln -s ../../bin/busybox ${xtroot}/usr/bin/chvt		#MISSING
	#ln -s ../../bin/busybox ${xtroot}/usr/bin/cmp		#MISSING
	#ln -s ../../bin/busybox ${xtroot}/usr/bin/length	#MISSING
	#ln -s ../../bin/busybox ${xtroot}/usr/bin/loadacm	#MISSING

	# Prevent the use of update-alternatives
	#echo "#!/bin/sh" > ${xtroot}/usr/sbin/update-alternatives
	#chmod 755 ${xtroot}/usr/sbin/update-alternatives

	# Do symlink to files and directories that are in ramdisk
	# May need to change some paths in init scripts for access to ramdisk files
	#links =  ["/mnt",		\
	#	  "/var/lib/xkb", 	\
	#	  "/var/run", 		\
	#	  "/var/log",		\
	#	  "/var/lock",		\
	#	  "/var/yp"]

	#if ramdir[0] is "/":
	#	ramdir = ramdir[1:]
		
	#for link in links:
	#	if link[0] is "/":
	#		link = link[1:]
	#	create_file(os.path.join(xtroot,ramdir,link))
	#	mklink(xtroot,os.path.join(ramdir,link),link,force=True)


	#touch ${xtroot}/tmp/mnt
	#touch ${xtroot}/tmp/var
	#touch ${xtroot}/tmp/hosts
	#touch ${xtroot}/tmp/initrunlvl
	#touch ${xtroot}/tmp/ioctl.save
	#touch ${xtroot}/tmp/mtab
	#touch ${xtroot}/tmp/resolv.conf
	#touch ${xtroot}/tmp/syslog.conf

	# Link of startx : we provide our own
	mklink(xtroot,"/etc/screen.d/startx","/bin/startx")
	# Specify wich X server to use
	mklink(xtroot,"/usr/X11R6/bin/Xorg","/etc/X11/X")

	# Configure SSH to permit root login
	if os.path.isfile(os.path.join(xtroot,"etc","ssh","sshd_config")):
		print "configuring SSH to accept root login"
		sh("sed -i -e 's/^PermitRootLogin.*$/PermitRootLogin yes/g' "+xtroot+"/etc/ssh/sshd_config")
		sh("sed -i -e 's/.*UsePAM.*$/UsePAM yes/g' "+xtroot+"/etc/ssh/sshd_config")
		sh("echo > "+xtroot+"/etc/ssh/denyusers")
	
	# Don't automatically load ipv6: we'll us modprobe if we need it
	sh("echo 'alias net-pf-10 off' >> "+xtroot+"/etc/modprobe.conf")

	# Create ssh keys for SSH
	# FIXME : new way to manage ssh keys?
	# ./mille-xterm-keygen ${xtroot}

	print """
Done! The xtermroot has been generated in %s

Initrd must now be build with the following command :
   
   mille-xterm-bootserver build_initrd

You can add kernels used by the initrd of terminals with the
following command, by replacing KERNEL_NAME by the choosen kernel :

   urpmi <KERNEL_NAME> --root=%s

Note : If your boot server has never been configured, you can do it now
with this command :

   mille-xterm-bootserver autoconfigure
   
This command will :
 - Enable the NFS export of the xtermroot
 - Select the configuration system (configurator or global lts.conf file)
 - Configure and start the NFS daemon

""" % (xtroot, xtroot)

if __name__=="__main__":
	print "This command will generate the xtermroot (the files system used by the operating system of terminals)."
	build()
