#/bin/bash
# Copyright 2004, Revolution Linux Inc., Alban Crequy
#
# This file is part of the MILLE-XTERM distribution.
# See the MILLE-XTERM (english) and/or the MILLE (french) project web site
#
# http://www.revolutionlinux.com/mille-xterm/
# http://www.mille.ca/
#
# The MILLE-XTERM framework is covered by the GNU General Public License. See
# the COPYING file in the top-level MILLE-XTERM directory. Software packages
# that are included in the MILLE-XTERM distribution have their own licenses.
#
# -------------------------------------------------------------------------


MOUNTPOINT=/tmp/part1

usage() {
	echo "$1"
	echo 'Usage:'
	echo '  init_doc.sh /dev/hdc images/<kernelversion>'
	exit 1
}

. checks.sh

# Find grub files
DIRNAME=`dirname $0`
if [ -r ${DIRNAME}/grub/stage1 ] ; then
	DIR_GRUB=${DIRNAME}/grub
	GRUB_EXE=$DIR_GRUB/grub
else
	usage "Cannot find grub installation"
fi

# Check if the boot directory exists
if [ -f "$2/vmlinuz" ] ; then
	REP_BOOT=$2
else
	usage "\"$2/vmlinuz\" does not exist"
fi
if [ -f "$2/initrd" ] ; then
	REP_BOOT=$2
else
	usage "\"$2/initrd\" does not exist"
fi

# Parameters
echo "DEVICE = $DEVICE"
echo "DEV_GRUB = $DEV_GRUB"
echo "DEV_GRUB1 = $DEV_GRUB1"

# Umount the disk on chip, if mounted... (maybe the admin forget to umount it)
umount ${DEVICE}1 &> /dev/null

mkdir -p $MOUNTPOINT

# Mount new partition
echo "Mount $DEVICE ..."
FAILED=0; rm -f /tmp/FAILED >/dev/null 2>&1
mount ${DEVICE}1 $MOUNTPOINT || FAILED=1

if [ $FAILED -eq 1 ]; then
 echo;echo;echo;echo
 echo "Hard drive not formatted or disk error"
 echo "Try manual installation"
 (for i in 1 2 3 4 5 6 7 8 9 10;do echo -ne "\007";sleep 1;done)&
 touch /tmp/FAILED
 exit 1
fi

# Copy files on new partition
echo "Copy /grub on ${DEVICE}1 ..."
mkdir -p $MOUNTPOINT/grub
cp -a $DIR_GRUB/stage1 $MOUNTPOINT/grub/
cp -a $DIR_GRUB/stage2 $MOUNTPOINT/grub/
cp -a $DIR_GRUB/e2fs_stage1_5 $MOUNTPOINT/grub/
cp -a $DIR_GRUB/fat15 $MOUNTPOINT/grub/
cp -a $DIR_GRUB/splash.xpm.gz $MOUNTPOINT/grub/

cat init_doc-menu-fat.lst > $MOUNTPOINT/grub/menu.lst

if [ -f $MOUNTPOINT/command.com -o -f $MOUNTPOINT/boot.ini ];then
cat << EOF >> $MOUNTPOINT/grub/menu.lst
title Windows/Dos
root (hd0,0)
makeactive
chainloader +1
boot
EOF
fi

cat << EOF > $MOUNTPOINT/grub/default
0
#
#
#
#
#
#
#
#
#
#
# WARNING: If you want to edit this file directly, do not remove any line
# from this file, including this warning. Using grub-set-default is
# strongly recommended.
EOF

echo "Copy /boot on ${DEVICE}1 ..."
mkdir -p $MOUNTPOINT/boot/default
mkdir -p $MOUNTPOINT/boot/test
cp -a $REP_BOOT/vmlinuz $MOUNTPOINT/boot/default
cp -a $REP_BOOT/initrd $MOUNTPOINT/boot/default
cp -a $REP_BOOT/version $MOUNTPOINT/boot/default
if [ ! -f $MOUNTPOINT/boot/default/version ]; then
  echo "0" > $MOUNTPOINT/boot/default/version
fi
cp -a $REP_BOOT/vmlinuz $MOUNTPOINT/boot/test
cp -a $REP_BOOT/initrd $MOUNTPOINT/boot/test
cp -a $REP_BOOT/version $MOUNTPOINT/boot/test
if [ ! -f $MOUNTPOINT/boot/test/version ]; then
  echo "0" > $MOUNTPOINT/boot/test/version
fi

echo "Copy /info on ${DEVICE}1 ..."
mkdir -p $MOUNTPOINT/info/
cp "$0" $MOUNTPOINT/info/init_doc.sh
date > $MOUNTPOINT/info/creation
sync

# Umount new partition
echo "Umount $DEVICE ..."
umount $MOUNTPOINT
rmdir $MOUNTPOINT
sync

# Grub
echo "Installing grub on $DEVICE ..."
sleep 1
$GRUB_EXE --no-floppy --batch << EOF
find /grub/stage1
root $DEV_GRUB1
setup $DEV_GRUB
quit
EOF

# done
echo "Done."


