#!/bin/sh
# Copyright 2006, CRIM, Revolution Linux Inc.
#
# $Id$
#
# Authors : Gabriel Lefebvre <Email>
#
# This file is part of the MILLE-XTERM project.
#
#       http://www.revolutionlinux.com/mille-xterm/
#       http://www.mille.ca/
#
# This program is covered by the GNU General Public License.
# See the COPYING file in the top-level MILLE-XTERM directory.
#
# ------------------------------------------------------------------------
#
# inventory.sh - a shell script for gathering information on a terminal's hardware

#     REQUIRES:  lspci, dmidecode
#
# 	 USAGE:  # [dmidecode] | inventory.sh [-d] <output_file>
#	         OR
		 # inventory.sh <output_file>

if [ "x"$2 == "x" ] ; then
	OUTPUT=$1
else
	USE_DMI=$1
	OUTPUT=$2
fi	

# Set where to redirect error messages :
ERROR=/dev/stdout

#
# Processing of dmidecode's output...
#
function dmi {
  SECTION=""
  EXIT=0
  
   while read line ; do # Loop that extracts the right information from dmidecode's output

	#JMD: rewrite parsing to use DMI type, since
	#the handle numbers are not the same on different platforms

        if [ x`echo $line|grep -qc "DMI type" && echo DMI` = xDMI ]; then
	    #echo "Found DMI Line"
	    type=`echo $line|cut -f2|cut -f1 -d,`
    	    if [ "$type" = "DMI type 0" ]; then
		SECTION="BIOS"
	    elif [ "$type" = "DMI type 1" ]; then
		SECTION="SYSTEM"
	    elif [ "$type" = "DMI type 2" ]; then
		SECTION="BASEBOARD"
	    elif [ "$type" = "DMI type 3" ]; then
		SECTION=""
	        echo -e "BIOS_VENDOR=$BIOS_VENDOR\nBIOS_VERSION=$BIOS_VERSION\nBIOS_RELDATE=$BIOS_RELDATE\nSYSTEM_MANUF=$SYSTEM_MANUF\nSYSTEM_PRODNAME=$SYSTEM_PRODNAME\nSYSTEM_VERSION=$SYSTEM_VERSION\nBASEBOARD_MANUF=$BASEBOARD_MANUF\nBASEBOARD_PRODNAME=$BASEBOARD_PRODNAME" >> $OUTPUT
	        break
		EXIT=1 # Done gathering information: echo and exit loop
	    fi	
        else
	    #echo "NOT DMI"
	    case $SECTION in 
	    BIOS)
		BIOS_VERSION_TEMP=`echo $line | awk '/Version:/ { print $0 }' | sed -e 's/.*Version: //'`
		BIOS_VENDOR_TEMP=`echo $line | awk '/Vendor:/ { print $0 }' | sed -e 's/.*Vendor: //'`
		BIOS_RELDATE_TEMP=`echo $line | awk '/Release Date:/ { print $0 }' | sed -e 's/.*Release Date: //'`

		if [ ! "$BIOS_VERSION_TEMP" = "" ] ; then
			BIOS_VERSION=$BIOS_VERSION_TEMP
		fi
		if [ ! "$BIOS_VENDOR_TEMP" = "" ] ; then
			BIOS_VENDOR=$BIOS_VENDOR_TEMP
		fi
		if [ ! "$BIOS_RELDATE_TEMP" = "" ] ; then
			BIOS_RELDATE=$BIOS_RELDATE_TEMP
		fi
	    ;;
	    SYSTEM)	
		SYSTEM_MANUF_TEMP=`echo $line | awk '/Manufacturer:/ { print $0 }' | sed -e 's/.*Manufacturer: //'`
		SYSTEM_PRODNAME_TEMP=`echo $line | awk '/Product Name:/ { print $0 }' | sed -e 's/.*Product Name: //'`
		SYSTEM_VERSION_TEMP=`echo $line | awk '/Version:/ { print $0 }' | sed -e 's/.*Version: //'`
		 
		if [ ! "$SYSTEM_MANUF_TEMP" = "" ] ; then
			SYSTEM_MANUF=$SYSTEM_MANUF_TEMP
		fi
		if [ ! "$SYSTEM_PRODNAME_TEMP" = "" ] ; then
			SYSTEM_PRODNAME=$SYSTEM_PRODNAME_TEMP
		fi
		if [ ! "$SYSTEM_VERSION_TEMP" = "" ] ; then
			SYSTEM_VERSION=$SYSTEM_VERSION_TEMP
		fi
	    ;;
	    BASEBOARD)
		BASEBOARD_MANUF_TEMP=`echo $line | awk '/Manufacturer:/ { print $0 }' | sed -e 's/.*Manufacturer: //'`
		BASEBOARD_PRODNAME_TEMP=`echo $line | awk '/Product Name:/ { print $0 }' | sed -e 's/.*Product Name: //'`

		if [ ! "$BASEBOARD_MANUF_TEMP" = "" ] ; then
			BASEBOARD_MANUF=$BASEBOARD_MANUF_TEMP
		fi
		if [ ! "$BASEBOARD_PRODNAME_TEMP" = "" ] ; then
			BASEBOARD_PRODNAME=$BASEBOARD_PRODNAME_TEMP
		fi
	    ;;
	    esac
	fi
	#if [ "$EXIT" = 1 ]; then	# Send all information to output & exit loop
	#    echo -e "BIOS_VENDOR=$BIOS_VENDOR\nBIOS_VERSION=$BIOS_VERSION\nBIOS_RELDATE=$BIOS_RELDATE\nSYSTEM_MANUF=$SYSTEM_MANUF\nSYSTEM_PRODNAME=$SYSTEM_PRODNAME\nSYSTEM_VERSION=$SYSTEM_VERSION\nBASEBOARD_MANUF=$BASEBOARD_MANUF\nBASEBOARD_PRODNAME=$BASEBOARD_PRODNAME" >> $OUTPUT
	#    break
	#fi
   done 
}

#
# Get CPU, memory and ide information by querying /proc
#
function memcpu {
	if [ -f /proc/meminfo ]; then
		echo "MEMTOTAL=`cat /proc/meminfo | grep MemTotal | tr -s " " "," | cut -d"," -f2`" >> $OUTPUT
	else echo -e "     /proc/meminfo : not found" >> $ERROR
	fi

	if [ -f /proc/cpuinfo ]; then
		echo "CPUFREQ=`cat /proc/cpuinfo  | grep "cpu MHz" | tail -1 | cut -d":" -f2`" | sed -e 's/CPUFREQ= /CPUFREQ=/g' >> $OUTPUT
		echo "CPUNAME=`cat /proc/cpuinfo  | grep "model name" | tail -1 | cut -d":" -f2`" | sed -e 's/CPUNAME= /CPUNAME=/g' >> $OUTPUT
		echo "CPUCOUNT=`cat /proc/cpuinfo | grep "physical id" | sort | uniq | wc -l`"  >> $OUTPUT
		echo "CPUVENDOR=`cat /proc/cpuinfo| grep "vendor_id" | tail -1 | cut -d":" -f2`" | sed -e 's/CPUVENDOR= /CPUVENDOR=/g'>> $OUTPUT
   		else echo -e "     /proc/cpuinfo : not found" >> $ERROR
   	fi	
}


function ide {
	for x in `ls /proc/ide/ | grep ide`; do
		
		if [ ! -d /proc/ide ]; then
			echo -e "     /proc/ide : not found" >> $ERROR
   			break
  		fi
		
		for y in `ls /proc/ide/$x/ | grep hd`; do
				echo -e "IDE_$y=$x" >> $OUTPUT
				echo -e "MEDIA_$y=`cat /proc/ide/$x/$y/media`" >> $OUTPUT
				echo -e "MODEL_$y=`cat /proc/ide/$x/$y/model`" >> $OUTPUT

				if [ -f /proc/ide/$x/$y/capacity ]; then
					echo -e "CAPACITY_$y=`cat /proc/ide/$x/$y/capacity`" >> $OUTPUT
				fi
  		done;
		
	done;
}
#
# Use 'lspci' to get all pci id's
#
function pciid {
	if [ -f /usr/bin/lspci ]; then
		for line in `lspci -n | cut -d" " -f4 | sort | uniq`; do
			echo "PCIID=$line" >> $OUTPUT
		done;
	else echo -e "     /usr/bin/lspci : not found" >> $ERROR
	fi
}

	#   Main   #


	if [ "x"$OUTPUT = "x" ] ; then
		echo "Output file not specified"
		exit
	fi

	cat /dev/null > $OUTPUT	# Empty the output file before filling it again with:
	if [ "x"$USE_DMI = "x-d" ] ; then
		dmi
	fi
	memcpu		# - memory & cpu info
	pciid		# - pci id numbers
	ide		# - ide info

