#/!bin/bash
# Copyright 2006, CRIM, Revolution Linux Inc.
#
# $Id: sshsecurity-cron.sh 1685 2006-04-01 05:00:47Z pgauvin $
#
# Authors : Gabriel Lefebvre <Email>
#
# This file is part of the MILLE-XTERM project.
#
#       http://www.revolutionlinux.com/mille-xterm/
#       http://www.mille.ca/
#
# This program is covered by the GNU General Public License.
# See the COPYING file in the top-level MILLE-XTERM directory.
#
# -------------------------------------------------------------------------
# /usr/sbin/sshsecurity-cron.sh
#
# sshsecurity-cron.sh should be called by crond at every minute.
# it retrieves 'security.txt' from the Configurator and uses it to
# manage the authorization of certain ssh keys and IP adresses. 

#
# Suck in some common LTSP functions
#
. /etc/ltsp_functions

WRKDIR=/root/.ssh
USERKEYFILE=user.pub
IPTABLES=/sbin/iptables
MY_CONFIG_SERVER=`cat /etc/config-server`
MY_NODE_IP="`/sbin/ifconfig eth0 | grep inet | cut -f2 -d":" | cut -f1 -d" "`"

SECUR_URL=$MY_CONFIG_SERVER/Security.php\?ip=$MY_NODE_IP 1>&2
SECUR_FILE=/tmp/security.txt

#
# Load security policies from configurator
#
rm -f $SECUR_FILE
wget -O $SECUR_FILE $SECUR_URL

WGET_KEYS=`grep key: $SECUR_FILE | sed -e 's/key://'`
WGET_IPS=`grep ip: $SECUR_FILE  | sed -e 's/ip://'`

#
# Save content into authorized_keys.cron file
#
echo $WGET_KEYS > $WRKDIR/authorized_keys.cron

#
# Backup existing authorized_keys to .orig files
#
for AUTHORIZED_KEY_FILE in "authorized_keys" "authorized_keys2"; do
        if [ ! -f $WRKDIR/$AUTHORIZED_KEY_FILE.orig ]; then
                if [ ! -f $WRKDIR/$AUTHORIZED_KEY_FILE ]; then
                        cat /dev/null > $WRKDIR/$AUTHORIZED_KEY_FILE.orig
                else
                        cp $WRKDIR/$AUTHORIZED_KEY_FILE $WRKDIR/$AUTHORIZED_KEY_FILE.orig
                fi
        fi
done


#
# Add $WGET_KEYS to the authorized_keys files
#
for AUTHORIZED_KEY_FILE in "authorized_keys" "authorized_keys2"; do
        cat $WRKDIR/$AUTHORIZED_KEY_FILE.orig > $WRKDIR/$AUTHORIZED_KEY_FILE
        if [ -f $WRKDIR/authorized_keys.session ]; then
                cat $WRKDIR/authorized_keys.session >> $WRKDIR/$AUTHORIZED_KEY_FILE
        fi
	cat $WRKDIR/authorized_keys.cron >> $WRKDIR/$AUTHORIZED_KEY_FILE
done


#
# Set up firewall rules that allow incoming connections
# from the IP adresses contained in security.txt
#
ENABLE_FIREWALL=`get_cfg ENABLE_FIREWALL none`
if [ "$ENABLE_FIREWALL" = "Y" ] ; then
	# flush the rules
	$IPTABLES -F TEMP
	
	# renew the rules	 
	for ip in $WGET_IPS ; do
		$IPTABLES -A TEMP -s $ip -j ACCEPT
	done 
fi

